"""A module for caching function results that contain unique characters in string."""

from collections import Counter
from functools import cache
import argparse

@cache
def unique_counter(s: str):
    """Return the number of characters that occur only once in a string."""
    char_counts = Counter(s)
    return sum(map(lambda count: count == 1, char_counts.values()))


def main():
    """Create a parser object with two arguments: --string and --file.
    Parses input arguments, determines which source to use (file or string),
    calls the function to count unique characters, and prints the result."""
    parser = argparse.ArgumentParser(description='Count unique characters in a string or a text file.')
    parser.add_argument("--string", type=str, help="Input string to process.")
    parser.add_argument("--file", type=str, help="Path to the file to process.")

    args = parser.parse_args()
    # Determine priority: if --file is provided, use it.
    if args.file:
        try:
            # Try to open and read the file contents.
            with open(args.file, "r", encoding="utf-8") as f:
                content = f.read()
        except FileNotFoundError:
            # Handle the case when the file does not exist.
            print(f"File not found: {args.file}")
            return

    # If no file provided, but --string is provided, use it.
    elif args.string:
        content = args.string

    # If neither --file nor --string is provided, show a message and exit.
    else:
        print("Please enter a string or a text file.")
        return

    # Call the function to count unique characters appearing only once.
    count = unique_counter(content)

    # Print the result.
    print(f"Number of unique characters occurring only once: {count}")
